@@ -26,6 +26,7 @@
 #endif
 
 #include <string>
+#include <gio/gio.h>
 #include <ibus.h>
 #include <pinyin.h>
 #include "PYUtil.h"
@@ -33,32 +34,30 @@
 
 namespace PY {
 
-class Bus;
-
-class Config : public Object {
+class Config {
 protected:
-    Config (Bus & bus, const std::string & name);
+    Config (const std::string & name);
     virtual ~Config (void);
 
 public:
     std::string dictionaries (void) const       { return m_dictionaries; }
+    std::string luaConverter (void) const       { return m_lua_converter; }
     pinyin_option_t option (void) const         { return m_option & m_option_mask; }
     guint orientation (void) const              { return m_orientation; }
     guint pageSize (void) const                 { return m_page_size; }
     gboolean rememberEveryInput (void) const    { return m_remember_every_input; }
     sort_option_t sortOption (void) const       { return m_sort_option; }
+    gboolean showSuggestion (void) const        { return m_show_suggestion; }
     gboolean shiftSelectCandidate (void) const  { return m_shift_select_candidate; }
     gboolean minusEqualPage (void) const        { return m_minus_equal_page; }
     gboolean commaPeriodPage (void) const       { return m_comma_period_page; }
     gboolean autoCommit (void) const            { return m_auto_commit; }
     gboolean doublePinyin (void) const          { return m_double_pinyin; }
     DoublePinyinScheme doublePinyinSchema (void) const { return m_double_pinyin_schema; }
-    gboolean doublePinyinShowRaw (void) const   { return m_double_pinyin_show_raw; }
     gboolean initChinese (void) const           { return m_init_chinese; }
     gboolean initFull (void) const              { return m_init_full; }
     gboolean initFullPunct (void) const         { return m_init_full_punct; }
     gboolean initSimpChinese (void) const       { return m_init_simp_chinese; }
-    gboolean specialPhrases (void) const        { return m_special_phrases; }
     ZhuyinScheme bopomofoKeyboardMapping (void) const   { return m_bopomofo_keyboard_mapping; }
     gint selectKeys (void) const                { return m_select_keys; }
     gboolean guideKey (void) const              { return m_guide_key; }
@@ -70,6 +69,7 @@ public:
     std::string letterSwitch (void) const       { return m_letter_switch; }
     std::string punctSwitch (void) const        { return m_punct_switch; }
     std::string tradSwitch (void) const         { return m_trad_switch; }
+    std::string openccConfig (void) const       { return m_opencc_config; }
 
 protected:
     bool read (const gchar * name, bool defval);
@@ -78,19 +78,20 @@ protected:
     void initDefaultValues (void);
 
     virtual void readDefaultValues (void);
-    virtual gboolean valueChanged (const std::string  &section,
+    virtual gboolean valueChanged (const std::string  &schema_id,
                                    const std::string  &name,
                                    GVariant           *value);
 private:
-    static void valueChangedCallback (IBusConfig     *config,
-                                      const gchar    *section,
+    static void valueChangedCallback (GSettings      *settings,
                                       const gchar    *name,
-                                      GVariant       *value,
                                       Config         *self);
 
 protected:
-    std::string m_section;
+    GSettings *m_settings;
+    std::string m_schema_id;
     std::string m_dictionaries;
+    std::string m_lua_converter;
+    std::string m_opencc_config;
     pinyin_option_t m_option;
     pinyin_option_t m_option_mask;
 
@@ -98,6 +99,7 @@ protected:
     guint m_page_size;
     gboolean m_remember_every_input;
     sort_option_t m_sort_option;
+    gboolean m_show_suggestion;
 
     gboolean m_shift_select_candidate;
     gboolean m_minus_equal_page;
@@ -106,13 +108,11 @@ protected:
 
     gboolean m_double_pinyin;
     DoublePinyinScheme m_double_pinyin_schema;
-    gboolean m_double_pinyin_show_raw;
 
     gboolean m_init_chinese;
     gboolean m_init_full;
     gboolean m_init_full_punct;
     gboolean m_init_simp_chinese;
-    gboolean m_special_phrases;
 
     ZhuyinScheme m_bopomofo_keyboard_mapping;
     gint m_select_keys;
@@ -133,24 +133,33 @@ protected:
 static inline bool
 normalizeGVariant (GVariant *value, bool defval)
 {
-    if (value == NULL || g_variant_classify (value) != G_VARIANT_CLASS_BOOLEAN)
+    if (value == NULL ||
+        g_variant_classify (value) != G_VARIANT_CLASS_BOOLEAN) {
+        g_warn_if_reached ();
         return defval;
+    }
     return g_variant_get_boolean (value);
 }
 
 static inline gint
 normalizeGVariant (GVariant *value, gint defval)
 {
-    if (value == NULL || g_variant_classify (value) != G_VARIANT_CLASS_INT32)
+    if (value == NULL ||
+        g_variant_classify (value) != G_VARIANT_CLASS_INT32) {
+        g_warn_if_reached ();
         return defval;
+    }
     return g_variant_get_int32 (value);
 }
 
 static inline std::string
 normalizeGVariant (GVariant *value, const std::string &defval)
 {
-    if (value == NULL || g_variant_classify (value) != G_VARIANT_CLASS_STRING)
+    if (value == NULL ||
+        g_variant_classify (value) != G_VARIANT_CLASS_STRING) {
+        g_warn_if_reached ();
         return defval;
+    }
     return g_variant_get_string (value, NULL);
 }
 
