@@ -79,7 +79,7 @@ BopomofoEditor::insert (gint ch)
 
 gboolean
 BopomofoEditor::processGuideKey (guint keyval, guint keycode,
-                                          guint modifiers)
+                                 guint modifiers)
 {
     if (!m_config.guideKey ())
         return FALSE;
@@ -166,7 +166,7 @@ BopomofoEditor::processSelectKey (guint keyval, guint keycode,
 
 gboolean
 BopomofoEditor::processBopomofo (guint keyval, guint keycode,
-                                          guint modifiers)
+                                 guint modifiers)
 {
     if (G_UNLIKELY (cmshm_filter (modifiers) != 0))
         return m_text ? TRUE : FALSE;
@@ -211,6 +211,22 @@ BopomofoEditor::processKeyEvent (guint keyval, guint keycode,
         m_select_mode = TRUE;
         return processSpace (keyval, keycode, modifiers);
 
+    case IBUS_Return:
+    case IBUS_KP_Enter:
+        /* no user input */
+        if (m_text.empty ())
+            return FALSE;
+
+        if (m_config.enterKey ())
+            commit ();
+        else {
+            Text text (m_text.c_str ());
+            commitText (text);
+        }
+
+        reset ();
+        return TRUE;
+
     case IBUS_Up:        case IBUS_KP_Up:
     case IBUS_Down:      case IBUS_KP_Down:
     case IBUS_Page_Up:   case IBUS_KP_Page_Up:
@@ -237,6 +253,41 @@ BopomofoEditor::processKeyEvent (guint keyval, guint keycode,
     return FALSE;
 }
 
+void
+BopomofoEditor::updateLookupTableLabel (void)
+{
+    String labels = bopomofo_select_keys[m_config.selectKeys ()];
+
+    size_t len = MIN (labels.length (), m_config.pageSize ());
+    for (size_t i = 0; i < len; ++i) {
+        String label = (gchar) labels[i];
+        Text text (label);
+        m_lookup_table.setLabel (i, text);
+    }
+}
+
+void
+BopomofoEditor::updateLookupTable (void)
+{
+    // needed by updatePreeditText
+    updateCandidates ();
+
+    if (!m_select_mode) {
+        hideLookupTable ();
+        return;
+    }
+
+    m_lookup_table.clear ();
+
+    fillLookupTable ();
+    updateLookupTableLabel ();
+    if (m_lookup_table.size()) {
+        Editor::updateLookupTable (m_lookup_table, TRUE);
+    } else {
+        hideLookupTable ();
+    }
+}
+
 void
 BopomofoEditor::updatePinyin (void)
 {
@@ -254,7 +305,7 @@ BopomofoEditor::updatePinyin (void)
 }
 
 void
-BopomofoEditor::commit (gint index)
+BopomofoEditor::commit (const gchar *str)
 {
     if (G_UNLIKELY (m_text.empty ()))
         return;
@@ -262,15 +313,56 @@ BopomofoEditor::commit (gint index)
     m_buffer.clear ();
 
     /* sentence candidate */
-    char *tmp = NULL;
-    pinyin_get_sentence (m_instance, index, &tmp);
-    if (tmp) {
-        if (m_props.modeSimp ()) {
-            m_buffer << tmp;
+    m_buffer << str;
+
+    /* text after pinyin */
+    const gchar *p = m_text.c_str() + m_pinyin_len;
+    while (*p != '\0') {
+        gchar ** symbols = NULL;
+        if (pinyin_in_chewing_keyboard (m_instance, *p, &symbols)) {
+            g_assert (1 == g_strv_length (symbols));
+            m_buffer << symbols[0];
+            g_strfreev (symbols);
         } else {
-            SimpTradConverter::simpToTrad (tmp, m_buffer);
+            if (G_UNLIKELY (m_props.modeFull ())) {
+                m_buffer.appendUnichar (HalfFullConverter::toFull (*p));
+            } else {
+                m_buffer << *p;
+            }
         }
-        g_free (tmp);
+        ++p;
+    }
+
+    Text text (m_buffer.c_str ());
+    commitText (text);
+
+    reset();
+}
+
+void
+BopomofoEditor::updatePreeditText ()
+{
+    guint num = 0;
+    pinyin_get_n_candidate (m_instance, &num);
+
+    /* preedit text = guessed sentence + un-parsed pinyin text */
+    if (G_UNLIKELY (m_text.empty () || 0 == num)) {
+        hidePreeditText ();
+        return;
+    }
+
+    m_buffer.clear ();
+
+    /* probe nbest match candidate */
+    lookup_candidate_type_t type;
+    lookup_candidate_t * candidate = NULL;
+    pinyin_get_candidate (m_instance, 0, &candidate);
+    pinyin_get_candidate_type (m_instance, candidate, &type);
+
+    gchar * sentence = NULL;
+    if (NBEST_MATCH_CANDIDATE == type) {
+        pinyin_get_sentence (m_instance, 0, &sentence);
+        m_buffer<<m_candidates[0].m_display_string;
     }
 
     /* text after pinyin */
@@ -291,38 +383,6 @@ BopomofoEditor::commit (gint index)
         ++p;
     }
 
-    pinyin_train(m_instance, index);
-    if (m_config.rememberEveryInput ())
-        LibPinyinBackEnd::instance ().rememberUserInput (m_instance, index);
-    LibPinyinBackEnd::instance ().modified();
-    PhoneticEditor::commit ((const gchar *)m_buffer);
-    reset();
-}
-
-void
-BopomofoEditor::updatePreeditText ()
-{
-    /* preedit text = guessed sentence + un-parsed pinyin text */
-    if (G_UNLIKELY (m_text.empty ())) {
-        hidePreeditText ();
-        return;
-    }
-
-    m_buffer.clear ();
-    char *sentence = NULL;
-    pinyin_get_sentence(m_instance, 0, &sentence);
-    if (sentence) {
-        if (m_props.modeSimp ()) {
-            m_buffer<<sentence;
-        } else {
-            SimpTradConverter::simpToTrad (sentence, m_buffer);
-        }
-    }
-
-    /* append rest text */
-    const gchar *p = m_text.c_str () + m_pinyin_len;
-    m_buffer << p;
-
     StaticText preedit_text (m_buffer);
     /* underline */
     preedit_text.appendAttribute (IBUS_ATTR_TYPE_UNDERLINE, IBUS_ATTR_UNDERLINE_SINGLE, 0, -1);
@@ -331,9 +391,6 @@ BopomofoEditor::updatePreeditText ()
     guint cursor = getPinyinCursor ();
     pinyin_get_character_offset(m_instance, sentence, cursor, &offset);
     Editor::updatePreeditText (preedit_text, offset, TRUE);
-
-    if (sentence)
-        g_free (sentence);
 }
 
 void
