@@ -22,9 +22,11 @@
  */
 #include "PYPBopomofoEngine.h"
 #include <string>
+#include <assert.h>
 #include "PYPunctEditor.h"
 #include "PYPBopomofoEditor.h"
 #include "PYFallbackEditor.h"
+#include "PYPSuggestionEditor.h"
 #include "PYConfig.h"
 #include "PYPConfig.h"
 
@@ -36,6 +38,7 @@ BopomofoEngine::BopomofoEngine (IBusEngine *engine)
       m_props (BopomofoConfig::instance ()),
       m_prev_pressed_key (IBUS_VoidSymbol),
       m_input_mode (MODE_INIT),
+      m_need_update (FALSE),
       m_fallback_editor (new FallbackEditor (m_props, BopomofoConfig::instance()))
 {
     gint i;
@@ -43,6 +46,7 @@ BopomofoEngine::BopomofoEngine (IBusEngine *engine)
     /* create editors */
     m_editors[MODE_INIT].reset (new BopomofoEditor (m_props, BopomofoConfig::instance ()));
     m_editors[MODE_PUNCT].reset (new PunctEditor (m_props, BopomofoConfig::instance ()));
+    m_editors[MODE_SUGGESTION].reset (new SuggestionEditor (m_props, BopomofoConfig::instance ()));
 
     m_props.signalUpdateProperty ().connect
         (std::bind (&BopomofoEngine::updateProperty, this, _1));
@@ -79,16 +83,22 @@ BopomofoEngine::processAccelKeyEvent (guint keyval, guint keycode,
         gboolean triggered = FALSE;
 
         if (m_prev_pressed_key == keyval) {
-            if (PinyinConfig::instance ().mainSwitch () == accel) {
+            if (BopomofoConfig::instance ().mainSwitch () == accel) {
                 triggered = TRUE;
             }
         }
 
         if (triggered) {
-            if (!m_editors[MODE_INIT]->text ().empty ())
+            if (!m_editors[MODE_INIT]->text ().empty ()) {
+                Text text (m_editors[MODE_INIT]->text ());
+                commitText (text);
                 m_editors[MODE_INIT]->reset ();
+            }
+
+            if (!m_editors[MODE_SUGGESTION]->text ().empty ())
+                m_editors[MODE_SUGGESTION]->reset ();
             m_props.toggleModeChinese ();
-            return TRUE;
+            return FALSE;
         }
 
         if (m_input_mode == MODE_INIT &&
@@ -102,21 +112,21 @@ BopomofoEngine::processAccelKeyEvent (guint keyval, guint keycode,
     }
 
     /* Toggle full/half Letter Mode */
-    if (PinyinConfig::instance (). letterSwitch () == accel) {
+    if (BopomofoConfig::instance (). letterSwitch () == accel) {
         m_props.toggleModeFull ();
         m_prev_pressed_key = keyval;
         return TRUE;
     }
 
     /* Toggle full/half Punct Mode */
-    if (PinyinConfig::instance (). punctSwitch () == accel) {
+    if (BopomofoConfig::instance (). punctSwitch () == accel) {
         m_props.toggleModeFullPunct ();
         m_prev_pressed_key = keyval;
         return TRUE;
     }
 
     /* Toggle simp/trad Chinese Mode */
-    if (PinyinConfig::instance ().tradSwitch () == accel) {
+    if (BopomofoConfig::instance ().tradSwitch () == accel) {
         m_props.toggleModeSimp ();
         m_prev_pressed_key = keyval;
         return TRUE;
@@ -141,6 +151,27 @@ BopomofoEngine::processKeyEvent (guint keyval, guint keycode, guint modifiers)
         return FALSE;
 
     if (m_props.modeChinese ()) {
+        /* return from MODE_SUGGESTION to normal input. */
+        if (m_input_mode == MODE_SUGGESTION) {
+            /* only accept input to select candidate. */
+            if (IBUS_Escape == keyval) {
+                m_editors[m_input_mode]->reset ();
+                m_input_mode = MODE_INIT;
+                m_editors[m_input_mode]->reset ();
+                /* m_editors[m_input_mode]->update (); */
+                return TRUE;
+            }
+
+            retval = m_editors[m_input_mode]->processKeyEvent (keyval, keycode, modifiers);
+
+            if (retval) {
+                goto out;
+            } else {
+                m_editors[m_input_mode]->reset ();
+                m_input_mode = MODE_INIT;
+            }
+        }
+
         if (G_UNLIKELY (m_input_mode == MODE_INIT &&
                         m_editors[MODE_INIT]->text ().empty () &&
                         cmshm_filter (modifiers) == 0 &&
@@ -160,6 +191,13 @@ BopomofoEngine::processKeyEvent (guint keyval, guint keycode, guint modifiers)
     if (G_UNLIKELY (!retval))
         retval = m_fallback_editor->processKeyEvent (keyval, keycode, modifiers);
 
+out:
+    /* needed for SuggestionEditor */
+    if (m_need_update) {
+        m_editors[m_input_mode]->update ();
+        m_need_update = FALSE;
+    }
+
     /* store ignored key event by editors */
     m_prev_pressed_key = retval ? IBUS_VoidSymbol : keyval;
 
@@ -260,8 +298,17 @@ void
 BopomofoEngine::commitText (Text & text)
 {
     Engine::commitText (text);
-    if (m_input_mode != MODE_INIT)
+
+    if (m_input_mode != MODE_INIT && m_input_mode != MODE_SUGGESTION) {
         m_input_mode = MODE_INIT;
+    } else if (BopomofoConfig::instance ().showSuggestion ()) {
+        m_input_mode = MODE_SUGGESTION;
+        m_editors[m_input_mode]->setText (text.text (), 0);
+        m_need_update = TRUE;
+    } else {
+        m_input_mode = MODE_INIT;
+    }
+
 #if 1
     /* handle "<num>+.<num>+" here */
     if (text.text ())
