@@ -18,28 +18,46 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+
 #include "PYPPhoneticEditor.h"
+#include <assert.h>
 #include "PYConfig.h"
 #include "PYPinyinProperties.h"
-#include "PYSimpTradConverter.h"
 
 using namespace PY;
 
 /* init static members */
 PhoneticEditor::PhoneticEditor (PinyinProperties &props,
-                                                  Config &config):
-    Editor (props, config),
+                                Config &config)
+    : Editor (props, config),
     m_pinyin_len (0),
-    m_lookup_table (m_config.pageSize ())
+    m_lookup_table (m_config.pageSize ()),
+    m_libpinyin_candidates (this),
+#ifdef IBUS_BUILD_LUA_EXTENSION
+    m_lua_trigger_candidates (this),
+    m_lua_converter_candidates (this),
+#endif
+    m_emoji_candidates (this),
+    m_traditional_candidates (this, config)
 {
 }
 
 PhoneticEditor::~PhoneticEditor (){
 }
 
+#ifdef IBUS_BUILD_LUA_EXTENSION
+gboolean
+PhoneticEditor::setLuaPlugin (IBusEnginePlugin *plugin)
+{
+    m_lua_trigger_candidates.setLuaPlugin (plugin);
+    m_lua_converter_candidates.setLuaPlugin (plugin);
+    return TRUE;
+}
+#endif
+
 gboolean
 PhoneticEditor::processSpace (guint keyval, guint keycode,
-                                       guint modifiers)
+                              guint modifiers)
 {
     if (!m_text)
         return FALSE;
@@ -73,10 +91,12 @@ PhoneticEditor::processFunctionKey (guint keyval, guint keycode, guint modifiers
     if (modifiers == 0) {  /* no modifiers. */
         switch (keyval) {
         case IBUS_Return:
-        case IBUS_KP_Enter:
-            commit (m_text.c_str ());
+        case IBUS_KP_Enter: {
+            Text text (m_text.c_str ());
+            commitText (text);
             reset ();
             return TRUE;
+        }
 
         case IBUS_BackSpace:
             removeCharBefore ();
@@ -192,6 +212,7 @@ PhoneticEditor::updateLookupTable (void)
 {
     m_lookup_table.clear ();
 
+    updateCandidates ();
     fillLookupTable ();
     if (m_lookup_table.size()) {
         Editor::updateLookupTable (m_lookup_table, TRUE);
@@ -200,75 +221,46 @@ PhoneticEditor::updateLookupTable (void)
     }
 }
 
-#if 0
 gboolean
-PhoneticEditor::fillLookupTableByPage (void)
+PhoneticEditor::updateCandidates (void)
 {
-    guint len = 0;
-    pinyin_get_n_candidate (m_instance, &len);
+    m_candidates.clear ();
 
-    guint filled_nr = m_lookup_table.size ();
-    guint page_size = m_lookup_table.pageSize ();
+    m_libpinyin_candidates.processCandidates (m_candidates);
 
-    /* fill lookup table by libpinyin get candidates. */
-    guint need_nr = MIN (page_size, len - filled_nr);
-    g_assert (need_nr >=0);
-    if (need_nr == 0)
-        return FALSE;
+    m_emoji_candidates.processCandidates (m_candidates);
 
-    String word;
-    for (guint i = filled_nr; i < filled_nr + need_nr; i++) {
-        if (i >= len)  /* no more candidates */
-            break;
+#ifdef IBUS_BUILD_LUA_EXTENSION
+    m_lua_trigger_candidates.processCandidates (m_candidates);
 
-        lookup_candidate_t * candidate = NULL;
-        pinyin_get_candidate (m_instance, i, &candidate);
+    std::string converter = m_config.luaConverter ();
 
-        const gchar * phrase_string = NULL;
-        pinyin_get_candidate_string (m_instance, candidate, &phrase_string);
-
-        /* show get candidates. */
-        if (G_LIKELY (m_props.modeSimp ())) {
-            word = phrase_string;
-        } else { /* Traditional Chinese */
-            word.truncate (0);
-            SimpTradConverter::simpToTrad (phrase_string, word);
-        }
-
-        Text text (word);
-        m_lookup_table.appendCandidate (text);
+    if (!converter.empty ()) {
+        m_lua_converter_candidates.setConverter (converter.c_str ());
+        m_lua_converter_candidates.processCandidates (m_candidates);
     }
+#endif
+
+    if (!m_props.modeSimp ())
+        m_traditional_candidates.processCandidates (m_candidates);
 
     return TRUE;
 }
-#endif
 
 gboolean
 PhoneticEditor::fillLookupTable (void)
 {
-    guint len = 0;
-    pinyin_get_n_candidate (m_instance, &len);
-
     String word;
-    for (guint i = 0; i < len; i++) {
-        lookup_candidate_t * candidate = NULL;
-        pinyin_get_candidate (m_instance, i, &candidate);
-
-        const gchar * phrase_string = NULL;
-        pinyin_get_candidate_string (m_instance, candidate, &phrase_string);
-
-        /* show get candidates. */
-        if (G_LIKELY (m_props.modeSimp ())) {
-            word = phrase_string;
-        } else { /* Traditional Chinese */
-            word.truncate (0);
-            SimpTradConverter::simpToTrad (phrase_string, word);
-        }
+    for (guint i = 0; i < m_candidates.size (); i++) {
+        EnhancedCandidate & candidate = m_candidates[i];
+        word = candidate.m_display_string;
 
         Text text (word);
+
         /* show user candidate as blue. */
-        if (pinyin_is_user_candidate (m_instance, candidate))
+        if (CANDIDATE_USER == candidate.m_candidate_type)
             text.appendAttribute (IBUS_ATTR_TYPE_FOREGROUND, 0x000000ef, 0, -1);
+
         m_lookup_table.appendCandidate (text);
     }
 
@@ -344,13 +336,6 @@ PhoneticEditor::update (void)
     updateAuxiliaryText ();
 }
 
-void
-PhoneticEditor::commit (const gchar *str)
-{
-    StaticText text(str);
-    commitText (text);
-}
-
 guint
 PhoneticEditor::getPinyinCursor ()
 {
@@ -372,6 +357,35 @@ PhoneticEditor::getLookupCursor (void)
     return lookup_cursor;
 }
 
+int
+PhoneticEditor::selectCandidateInternal (EnhancedCandidate & candidate)
+{
+    switch (candidate.m_candidate_type) {
+    case CANDIDATE_NBEST_MATCH:
+    case CANDIDATE_NORMAL:
+    case CANDIDATE_USER:
+        return m_libpinyin_candidates.selectCandidate (candidate);
+
+    case CANDIDATE_TRADITIONAL_CHINESE:
+        return m_traditional_candidates.selectCandidate (candidate);
+
+#ifdef IBUS_BUILD_LUA_EXTENSION
+    case CANDIDATE_LUA_TRIGGER:
+        return m_lua_trigger_candidates.selectCandidate (candidate);
+
+    case CANDIDATE_LUA_CONVERTER:
+        return m_lua_converter_candidates.selectCandidate (candidate);
+#endif
+
+    case CANDIDATE_EMOJI:
+        return m_emoji_candidates.selectCandidate (candidate);
+
+    default:
+        assert (FALSE);
+    }
+}
+
+#if 0
 gboolean
 PhoneticEditor::selectCandidate (guint i)
 {
@@ -418,6 +432,25 @@ PhoneticEditor::selectCandidate (guint i)
     update ();
     return TRUE;
 }
+#endif
+
+gboolean
+PhoneticEditor::selectCandidate (guint index)
+{
+    if (G_UNLIKELY (index >= m_candidates.size ()))
+        return FALSE;
+
+    EnhancedCandidate & candidate = m_candidates[index];
+    int action = selectCandidateInternal (candidate);
+
+    if (action & SELECT_CANDIDATE_COMMIT)
+        commit (candidate.m_display_string.c_str ());
+
+    if (action & SELECT_CANDIDATE_UPDATE)
+        update ();
+
+    return TRUE;
+}
 
 gboolean
 PhoneticEditor::selectCandidateInPage (guint i)
