@@ -21,14 +21,30 @@
 #ifndef __PY_LIB_PINYIN_BASE_EDITOR_H_
 #define __PY_LIB_PINYIN_BASE_EDITOR_H_
 
+#ifdef IBUS_BUILD_LUA_EXTENSION
+#include "lua-plugin.h"
+#endif
+
+#include <vector>
 #include <pinyin.h>
 #include "PYLookupTable.h"
 #include "PYEditor.h"
+#include "PYPEnhancedCandidates.h"
+#include "PYPLibPinyinCandidates.h"
+#include "PYPTradCandidates.h"
 
+#ifdef IBUS_BUILD_LUA_EXTENSION
+#include "PYPLuaTriggerCandidates.h"
+#include "PYPLuaConverterCandidates.h"
+#endif
+
+#include "PYPEmojiCandidates.h"
 
 namespace PY {
 
 class PhoneticEditor : public Editor {
+    friend class LibPinyinCandidates;
+
 public:
     PhoneticEditor (PinyinProperties & props, Config & config);
     virtual ~PhoneticEditor ();
@@ -47,13 +63,21 @@ public:
     virtual gboolean processFunctionKey (guint keyval, guint keycode, guint modifiers);
     virtual void updateLookupTable ();
     virtual void updateLookupTableFast ();
+    virtual gboolean updateCandidates ();
     virtual gboolean fillLookupTable ();
+    virtual void commit (const gchar *str) = 0;
+
+#ifdef IBUS_BUILD_LUA_EXTENSION
+    gboolean setLuaPlugin (IBusEnginePlugin *plugin);
+#endif
 
 protected:
+    virtual int selectCandidateInternal (EnhancedCandidate & candidate);
     gboolean selectCandidate (guint i);
     gboolean selectCandidateInPage (guint i);
 
-    void commit (const gchar *str);
+    void commit () { selectCandidate (0); }
+
     guint getPinyinCursor (void);
     guint getLookupCursor (void);
 
@@ -71,7 +95,6 @@ protected:
     virtual gboolean moveCursorRightByWord (void);
     virtual gboolean moveCursorToBegin (void);
     virtual gboolean moveCursorToEnd (void);
-    virtual void commit (gint index = 0) = 0;
     virtual void updateAuxiliaryText (void) = 0;
     virtual void updatePreeditText (void) = 0;
     virtual void updatePinyin (void) = 0;
@@ -87,6 +110,21 @@ protected:
 
     /* use LibPinyinBackEnd here. */
     pinyin_instance_t           *m_instance;
+
+    /* use EnhancedCandidates here. */
+    std::vector<EnhancedCandidate> m_candidates;
+
+    /* several EnhancedCandidates providers. */
+    LibPinyinCandidates m_libpinyin_candidates;
+
+#ifdef IBUS_BUILD_LUA_EXTENSION
+    LuaTriggerCandidates m_lua_trigger_candidates;
+    LuaConverterCandidates m_lua_converter_candidates;
+#endif
+
+    EmojiCandidates m_emoji_candidates;
+
+    TraditionalCandidates m_traditional_candidates;
 };
 
 };
