@@ -199,7 +199,7 @@ PinyinEditor::processKeyEvent (guint keyval, guint keycode,
 }
 
 void
-PinyinEditor::commit (gint index)
+PinyinEditor::commit (const gchar *str)
 {
     if (G_UNLIKELY (m_text.empty ()))
         return;
@@ -207,16 +207,7 @@ PinyinEditor::commit (gint index)
     m_buffer.clear ();
 
     /* sentence candidate */
-    char *tmp = NULL;
-    pinyin_get_sentence (m_instance, index, &tmp);
-    if (tmp) {
-        if (m_props.modeSimp ()) {
-            m_buffer << tmp;
-        } else {
-            SimpTradConverter::simpToTrad (tmp, m_buffer);
-        }
-        g_free (tmp);
-    }
+    m_buffer << str;
 
     /* text after pinyin */
     const gchar *p = m_text.c_str() + m_pinyin_len;
@@ -228,32 +219,36 @@ PinyinEditor::commit (gint index)
         m_buffer << p;
     }
 
-    pinyin_train (m_instance, index);
-    if (m_config.rememberEveryInput ())
-        LibPinyinBackEnd::instance ().rememberUserInput (m_instance, index);
-    LibPinyinBackEnd::instance ().modified ();
-    PhoneticEditor::commit ((const gchar *)m_buffer);
+    Text text (m_buffer.c_str ());
+    commitText (text);
+
     reset();
 }
 
 void
 PinyinEditor::updatePreeditText ()
 {
+    guint num = 0;
+    pinyin_get_n_candidate (m_instance, &num);
+
     /* preedit text = guessed sentence + un-parsed pinyin text */
-    if (G_UNLIKELY (m_text.empty ())) {
+    if (G_UNLIKELY (m_text.empty () || 0 == num)) {
         hidePreeditText ();
         return;
     }
 
     m_buffer.clear ();
-    char *sentence = NULL;
-    pinyin_get_sentence (m_instance, 0, &sentence);
-    if (sentence) {
-        if (m_props.modeSimp ()) {
-            m_buffer<<sentence;
-        } else {
-            SimpTradConverter::simpToTrad (sentence, m_buffer);
-        }
+
+    /* probe nbest match candidate */
+    lookup_candidate_type_t type;
+    lookup_candidate_t * candidate = NULL;
+    pinyin_get_candidate (m_instance, 0, &candidate);
+    pinyin_get_candidate_type (m_instance, candidate, &type);
+
+    gchar * sentence = NULL;
+    if (NBEST_MATCH_CANDIDATE == type) {
+        pinyin_get_sentence (m_instance, 0, &sentence);
+        m_buffer<<m_candidates[0].m_display_string;
     }
 
     /* append rest text */
@@ -268,9 +263,6 @@ PinyinEditor::updatePreeditText ()
     guint cursor = getPinyinCursor ();
     pinyin_get_character_offset(m_instance, sentence, cursor, &offset);
     Editor::updatePreeditText (preedit_text, offset, TRUE);
-
-    if (sentence)
-        g_free (sentence);
 }
 
 #if 0
