@@ -21,6 +21,7 @@
  */
 #include "PYPPinyinEngine.h"
 #include <string>
+#include <assert.h>
 #include "PYConfig.h"
 #include "PYPConfig.h"
 #include "PYPunctEditor.h"
@@ -37,6 +38,7 @@
 #include "PYPFullPinyinEditor.h"
 #include "PYPDoublePinyinEditor.h"
 #include "PYFallbackEditor.h"
+#include "PYPSuggestionEditor.h"
 
 using namespace PY;
 
@@ -46,18 +48,32 @@ PinyinEngine::PinyinEngine (IBusEngine *engine)
       m_props (PinyinConfig::instance ()),
       m_prev_pressed_key (IBUS_VoidSymbol),
       m_input_mode (MODE_INIT),
+      m_need_update (FALSE),
       m_fallback_editor (new FallbackEditor (m_props, PinyinConfig::instance ()))
 {
     gint i;
 
+#ifdef IBUS_BUILD_LUA_EXTENSION
+    initLuaPlugin ();
+#endif
+
     m_double_pinyin = PinyinConfig::instance ().doublePinyin ();
 
-    if (m_double_pinyin)
-        m_editors[MODE_INIT].reset
-            (new DoublePinyinEditor (m_props, PinyinConfig::instance ()));
-    else
-        m_editors[MODE_INIT].reset
-            (new FullPinyinEditor (m_props, PinyinConfig::instance ()));
+    if (m_double_pinyin) {
+        DoublePinyinEditor *editor = new DoublePinyinEditor
+            (m_props, PinyinConfig::instance ());
+        m_editors[MODE_INIT].reset (editor);
+#ifdef IBUS_BUILD_LUA_EXTENSION
+        editor->setLuaPlugin (m_lua_plugin);
+#endif
+    } else {
+        FullPinyinEditor *editor = new FullPinyinEditor
+            (m_props, PinyinConfig::instance ());
+        m_editors[MODE_INIT].reset (editor);
+#ifdef IBUS_BUILD_LUA_EXTENSION
+        editor->setLuaPlugin (m_lua_plugin);
+#endif
+    }
 
     m_editors[MODE_PUNCT].reset
         (new PunctEditor (m_props, PinyinConfig::instance ()));
@@ -65,7 +81,11 @@ PinyinEngine::PinyinEngine (IBusEngine *engine)
         (new RawEditor (m_props, PinyinConfig::instance ()));
 
 #ifdef IBUS_BUILD_LUA_EXTENSION
-    m_editors[MODE_EXTENSION].reset (new ExtEditor (m_props, PinyinConfig::instance ()));
+    {
+        ExtEditor *editor = new ExtEditor (m_props, PinyinConfig::instance ());
+        m_editors[MODE_EXTENSION].reset (editor);
+        editor->setLuaPlugin (m_lua_plugin);
+    }
 #else
     m_editors[MODE_EXTENSION].reset (new Editor (m_props, PinyinConfig::instance ()));
 #endif
@@ -80,6 +100,15 @@ PinyinEngine::PinyinEngine (IBusEngine *engine)
     m_editors[MODE_STROKE].reset (new Editor (m_props, PinyinConfig::instance ()));
 #endif
 
+    {
+        SuggestionEditor *editor = new SuggestionEditor
+            (m_props, PinyinConfig::instance ());
+        m_editors[MODE_SUGGESTION].reset (editor);
+#ifdef IBUS_BUILD_LUA_EXTENSION
+        editor->setLuaPlugin (m_lua_plugin);
+#endif
+    }
+
     m_props.signalUpdateProperty ().connect
         (std::bind (&PinyinEngine::updateProperty, this, _1));
 
@@ -95,6 +124,30 @@ PinyinEngine::~PinyinEngine (void)
 {
 }
 
+#ifdef IBUS_BUILD_LUA_EXTENSION
+gboolean
+PinyinEngine::initLuaPlugin (void)
+{
+    m_lua_plugin = ibus_engine_plugin_new ();
+
+    loadLuaScript ( ".." G_DIR_SEPARATOR_S "lua" G_DIR_SEPARATOR_S "base.lua")||
+        loadLuaScript (PKGDATADIR G_DIR_SEPARATOR_S "base.lua");
+
+    gchar * path = g_build_filename (g_get_user_config_dir (),
+                             "ibus", "libpinyin", "user.lua", NULL);
+    loadLuaScript(path);
+    g_free(path);
+
+    return TRUE;
+}
+
+gboolean
+PinyinEngine::loadLuaScript (const char * filename)
+{
+    return !ibus_engine_plugin_load_lua_script (m_lua_plugin, filename);
+}
+#endif
+
 /* keep synced with bopomofo engine. */
 gboolean
 PinyinEngine::processAccelKeyEvent (guint keyval, guint keycode,
@@ -121,8 +174,14 @@ PinyinEngine::processAccelKeyEvent (guint keyval, guint keycode,
         }
 
         if (triggered) {
-            if (!m_editors[MODE_INIT]->text ().empty ())
+            if (!m_editors[MODE_INIT]->text ().empty ()) {
+                Text text (m_editors[MODE_INIT]->text ());
+                commitText (text);
                 m_editors[MODE_INIT]->reset ();
+            }
+
+            if (!m_editors[MODE_SUGGESTION]->text ().empty ())
+                m_editors[MODE_SUGGESTION]->reset ();
             m_props.toggleModeChinese ();
             return FALSE;
         }
@@ -177,6 +236,28 @@ PinyinEngine::processKeyEvent (guint keyval, guint keycode, guint modifiers)
         return FALSE;
 
     if (m_props.modeChinese ()) {
+        /* return from MODE_SUGGESTION to normal input. */
+        if (m_input_mode == MODE_SUGGESTION) {
+            /* only accept input to select candidate. */
+            if (IBUS_Escape == keyval) {
+                m_editors[m_input_mode]->reset ();
+                m_input_mode = MODE_INIT;
+                m_editors[m_input_mode]->reset ();
+                /* m_editors[m_input_mode]->update ();*/
+                return TRUE;
+            }
+
+            retval = m_editors[m_input_mode]->processKeyEvent (keyval, keycode, modifiers);
+
+            if (retval) {
+                goto out;
+            } else {
+                m_editors[m_input_mode]->reset ();
+                m_input_mode = MODE_INIT;
+            }
+        }
+
+        /* handle normal input. */
         if (m_input_mode == MODE_INIT &&
             (cmshm_filter (modifiers) == 0)) {
             const String & text = m_editors[MODE_INIT]->text ();
@@ -242,6 +323,12 @@ PinyinEngine::processKeyEvent (guint keyval, guint keycode, guint modifiers)
     if (G_UNLIKELY (!retval))
         retval = m_fallback_editor->processKeyEvent (keyval, keycode, modifiers);
 
+out:
+    /* needed for SuggestionEditor */
+    if (m_need_update) {
+        m_editors[m_input_mode]->update ();
+        m_need_update = FALSE;
+    }
     /* store ignored key event by editors */
     m_prev_pressed_key = retval ? IBUS_VoidSymbol : keyval;
 
@@ -255,14 +342,24 @@ PinyinEngine::focusIn (void)
      *       or switch full/double pinyin when pinyin config is changed.*/
     if (PinyinConfig::instance ().doublePinyin ()) {
         if (!m_double_pinyin) {
-            m_editors[MODE_INIT].reset (new DoublePinyinEditor (m_props, PinyinConfig::instance ()));
+            DoublePinyinEditor *editor = new DoublePinyinEditor
+                (m_props, PinyinConfig::instance ());
+            m_editors[MODE_INIT].reset (editor);
+#ifdef IBUS_BUILD_LUA_EXTENSION
+            editor->setLuaPlugin (m_lua_plugin);
+#endif
             connectEditorSignals (m_editors[MODE_INIT]);
         }
         m_double_pinyin = TRUE;
     }
     else {
         if (m_double_pinyin) {
-            m_editors[MODE_INIT].reset (new FullPinyinEditor (m_props, PinyinConfig::instance ()));
+            FullPinyinEditor *editor = new FullPinyinEditor
+                (m_props, PinyinConfig::instance ());
+            m_editors[MODE_INIT].reset (editor);
+#ifdef IBUS_BUILD_LUA_EXTENSION
+            editor->setLuaPlugin (m_lua_plugin);
+#endif
             connectEditorSignals (m_editors[MODE_INIT]);
         }
         m_double_pinyin = FALSE;
@@ -356,8 +453,17 @@ void
 PinyinEngine::commitText (Text & text)
 {
     Engine::commitText (text);
-    if (m_input_mode != MODE_INIT)
+
+    if (m_input_mode != MODE_INIT && m_input_mode != MODE_SUGGESTION) {
         m_input_mode = MODE_INIT;
+    } else if (PinyinConfig::instance ().showSuggestion ()) {
+        m_input_mode = MODE_SUGGESTION;
+        m_editors[m_input_mode]->setText (text.text (), 0);
+        m_need_update = TRUE;
+    } else {
+        m_input_mode = MODE_INIT;
+    }
+
 #if 1
     /* handle "<num>+.<num>+" here */
     if (text.text ())
