@@ -22,6 +22,10 @@
 #ifndef __PY_LIB_PINYIN_PINYIN_ENGINE_H_
 #define __PY_LIB_PINYIN_PINYIN_ENGINE_H_
 
+#ifdef IBUS_BUILD_LUA_EXTENSION
+#include "lua-plugin.h"
+#endif
+
 #include "PYEngine.h"
 #include "PYPinyinProperties.h"
 
@@ -48,6 +52,11 @@ public:
     void candidateClicked (guint index, guint button, guint state);
 
 private:
+#ifdef IBUS_BUILD_LUA_EXTENSION
+    gboolean initLuaPlugin (void);
+    gboolean loadLuaScript (const char * filename);
+#endif
+
     gboolean processPunct (guint keyval, guint keycode, guint modifiers);
 
     void showSetupDialog (void);
@@ -67,14 +76,20 @@ private:
         MODE_ENGLISH,           // press v into English input mode
         MODE_STROKE,            // press u into stroke input mode
         MODE_EXTENSION,         // press i into extension input mode
+        MODE_SUGGESTION,        // suggestion mode
         MODE_LAST,
     } m_input_mode;
 
+    gboolean m_need_update;
+
     gboolean m_double_pinyin;
 
     EditorPtr m_editors[MODE_LAST];
     EditorPtr m_fallback_editor;
 
+#ifdef IBUS_BUILD_LUA_EXTENSION
+    Pointer<IBusEnginePlugin> m_lua_plugin;
+#endif
 };
 
 };
